/* Copyright (C) 2004   db4objects Inc.   http://www.db4o.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA  02111-1307, USA. */

package com.db4o;

import com.db4o.config.*;
import com.db4o.reflect.*;

class Config4Field extends Config4Abstract implements ObjectField, Cloneable, DeepClone {
    Config4Class i_class;
    IField i_fieldReflector;
    boolean i_queryEvaluation = true;
    int i_indexed = 0;
    MetaField i_metaField;
    boolean i_initialized;

    Config4Field(Config4Class a_class, String a_name) {
        i_class = a_class;
        i_name = a_name;
    }

    String className() {
        return i_class.getName();
    }

    public Object deepClone(Object param) throws CloneNotSupportedException {
        Config4Field ret = (Config4Field) clone();
        ret.i_class = (Config4Class) param;
        return ret;
    }

    Class invocationClass() throws Exception {
        return fieldReflector().getType();
    }

    private IField fieldReflector() {
        if (i_fieldReflector == null) {
            try {
                IClass classReflector = Db4o.reflector().forName(className());
                i_fieldReflector = classReflector.getDeclaredField(getName());
                i_fieldReflector.setAccessible();
            } catch (Exception e) {
            }
        }
        return i_fieldReflector;
    }

    public void queryEvaluation(boolean flag) {
        i_queryEvaluation = flag;
    }

    public void rename(String newName) {
        i_class.i_config.rename(new Rename(i_class.getName(), i_name, newName));
        i_name = newName;
    }

    public void indexed(boolean flag) {
        if (flag) {
            i_indexed = 1;
        } else {
            i_indexed = -1;
        }
    }

    public void initOnUp(Transaction systemTrans, YapField yapField) {
        if (!i_initialized) {
            YapStream anyStream = systemTrans.i_stream;
            if(Tuning.fieldIndices){
	            if (anyStream.maintainsIndices()) {
	                if(Debug.indexAllFields){
	                    i_indexed = 1;
	                }
	                if (! yapField.supportsIndex()) {
	                    i_indexed = -1;
	                }
	                
	                boolean indexInitCalled = false;
	                
	            	YapFile stream = (YapFile)anyStream;
	                i_metaField = i_class.i_metaClass.ensureField(systemTrans, i_name);
	                if (i_indexed == 1) {
	                    if (i_metaField.index == null) {
	                        i_metaField.index = new MetaIndex();
	                        stream.set3(systemTrans, i_metaField.index, YapConst.UNSPECIFIED, false);
	                        stream.set3(systemTrans, i_metaField, YapConst.UNSPECIFIED, false);
	                        yapField.initIndex(systemTrans, i_metaField.index);
	                        indexInitCalled = true;
	        				if (stream.i_config.i_messageLevel > YapConst.NONE) {
	        					stream.message("creating index " + yapField.toString());
	        				}
	        				YapClass yc = yapField.getParentYapClass();
	        				long[] ids = yc.getIDs();
	        				for (int i = 0; i < ids.length; i++) {
                                YapWriter writer = stream.readWriterByID(systemTrans, (int)ids[i]);
                                if(writer != null){
                                    Object obj = null;
                                    if(yc.findOffset(writer, yapField)){
    	                                try {
    	                                    obj = yapField.read(writer);
    	                                } catch (CorruptionException e) {
    	                                    if(Deploy.debug || Debug.atHome){
    	                                        e.printStackTrace();
    	                                    }
    	                                }
                                    }
                                    yapField.addIndexEntry(systemTrans, (int)ids[i], obj);
                                }else{
                                    if(Deploy.debug){
                                        throw new RuntimeException("Unexpected null object for ID");
                                    }
                                }
                            }
	        				if(ids.length > 0){
	        				    systemTrans.commit();
	        				}
	                    }
	                }
	                if (i_indexed == -1) {
	                    if (i_metaField.index != null) {
	        				if (stream.i_config.i_messageLevel > YapConst.NONE) {
	        					stream.message("dropping index " + yapField.toString());
	        				}
	                        MetaIndex mi = i_metaField.index;
	                        if (mi.indexLength > 0) {
	                            stream.free(mi.indexAddress, mi.indexLength);
	                        }
	                        if (mi.patchLength > 0) {
	                            stream.free(mi.patchAddress, mi.patchLength);
	                        }
	                        stream.delete1(systemTrans, mi);
	                        i_metaField.index = null;
	                        stream.setInternal(systemTrans, i_metaField, YapConst.UNSPECIFIED, false);
	                    }
	                }
	                if (i_metaField.index != null) {
	                    if(! indexInitCalled){
	                        yapField.initIndex(systemTrans, i_metaField.index);
	                    }
	                }
	            }
            }
            i_initialized = true;
        }
    }

}
