/* Copyright (C) 2004   db4objects Inc.   http://www.db4o.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA  02111-1307, USA. */

package com.db4o;

import com.db4o.reflect.*;

final class EventDispatcher
{
	private static final String[] events = {
		"objectCanDelete",
		"objectOnDelete", 
		"objectOnActivate", 
		"objectOnDeactivate",
		"objectOnNew",
		"objectOnUpdate",
		"objectCanActivate",
		"objectCanDeactivate",
		"objectCanNew",
		"objectCanUpdate"
	};
	
	static final int CAN_DELETE = 0;
	static final int DELETE = 1;
	static final int SERVER_COUNT = 2;
	static final int ACTIVATE = 2;
	static final int DEACTIVATE = 3;
	static final int NEW = 4;
	static final int UPDATE = 5;
	static final int CAN_ACTIVATE = 6;
	static final int CAN_DEACTIVATE = 7;
	static final int CAN_NEW = 8;
	static final int CAN_UPDATE = 9;
	static final int COUNT = 10;
	
	private final IMethod[] methods;
	
	private EventDispatcher(IMethod[] methods){
		this.methods = methods;
	}
	
	boolean dispatch(YapStream stream, Object obj, int eventID){
		if(methods[eventID] != null){
			Object[] parameters = new Object[]{stream};
			try{
				Object res = methods[eventID].invoke(obj,parameters);
				if(res != null && res instanceof Boolean){
				    return ((Boolean)res).booleanValue();
				}
			}catch(Throwable t){
			}
		}
		return true;
	}
	
	static EventDispatcher forClass(YapStream a_stream, IClass classReflector){
		EventDispatcher dispatcher = null;
		if(a_stream != null){
		    int count = 0;
		    if(a_stream.i_config.i_callbacks){
		        count = COUNT;
		    }else if(a_stream.i_config.i_isServer){
		        count = SERVER_COUNT;
		    }
		    if(count > 0){
				Class[] parameterClasses = {YapConst.CLASS_OBJECTCONTAINER};
				IMethod[] methods = new IMethod[COUNT];
				for (int i = COUNT -1; i >=0; i--){
					try{
						methods[i] = classReflector.getMethod(events[i], parameterClasses);
						if(dispatcher == null){
							dispatcher = new EventDispatcher(methods);
						}
					}catch(Throwable t){}
				}
		    }
		}
		return dispatcher;
	}
}
