/* Copyright (C) 2004   db4objects Inc.   http://www.db4o.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA  02111-1307, USA. */

package com.db4o;

/**
 * Index Path to represent a list of traversed index tree entries,
 * used by IxTraverser
 */
class IxPath implements Cloneable, Visitor4 {

    private QCandidates i_candidates;

    private Tree        i_candidatesTree;

    int                 i_comparisonResult;

    int[]               i_lowerAndUpperMatch;
    int                 i_upperNull = -1;

    IxPath              i_next;

    IxTraverser         i_traverser;
    IxTree              i_tree;

    IxPath(IxTraverser a_traverser, IxPath a_next, IxTree a_tree,
        int a_comparisonResult) {
        i_traverser = a_traverser;
        i_next = a_next;
        i_tree = a_tree;
        i_comparisonResult = a_comparisonResult;
        if (a_tree instanceof IxFileRange) {
            i_lowerAndUpperMatch = a_tree.i_fieldTransaction.i_index
                .fileRangeReader().lowerAndUpperMatches();
        }
    }

    Tree addPrecedingToCandidatesTree(Tree a_tree, QCandidates a_candidates) {
        i_candidatesTree = a_tree;
        i_candidates = a_candidates;
        if (i_tree.i_preceding != null) {
            if (i_next == null || i_next.i_tree != i_tree.i_preceding) {
                i_tree.i_preceding.traverse(this);
            }
        }
        if (i_lowerAndUpperMatch != null) {
            int[] lowerAndUpperMatch = new int[] { i_upperNull,
                i_lowerAndUpperMatch[0] - 1};
            i_candidatesTree = i_tree.addToCandidatesTree(i_candidatesTree,
                i_candidates, lowerAndUpperMatch);
        } else {
            if (i_comparisonResult < 0) {
                visit(i_tree);
            }
        }
        return i_candidatesTree;
    }

    Tree addSubsequentToCandidatesTree(Tree a_tree, QCandidates a_candidates) {
        i_candidatesTree = a_tree;
        i_candidates = a_candidates;
        if (i_tree.i_subsequent != null) {
            if (i_next == null || i_next.i_tree != i_tree.i_subsequent) {
                i_tree.i_subsequent.traverse(this);
            }
        }
        if (i_lowerAndUpperMatch != null) {
            int[] lowerAndUpperMatch = new int[] { i_lowerAndUpperMatch[1] + 1,
                ((IxFileRange) i_tree).i_entries - 1};
            i_candidatesTree = i_tree.addToCandidatesTree(i_candidatesTree,
                i_candidates, lowerAndUpperMatch);
        } else {
            if (i_comparisonResult > 0) {
                visit(i_tree);
            }
        }
        return i_candidatesTree;
    }

    Tree addToCandidatesTree(Tree a_tree, QCandidates a_candidates) {
        if (i_comparisonResult == 0 && i_traverser.i_take[1]) {
            a_tree = i_tree.addToCandidatesTree(a_tree, a_candidates,
                i_lowerAndUpperMatch);
        }
        return a_tree;
    }

    IxPath append(IxPath a_head, IxPath a_tail) {
        if (a_head == null) {
            return this;
        }
        i_next = a_head;
        return a_tail;
    }

    IxPath append(IxTree a_tree, int a_comparisonResult) {
        i_next = new IxPath(i_traverser, null, a_tree, a_comparisonResult);
        i_next.i_tree = a_tree;
        return i_next;
    }

    boolean carriesTheSame(IxPath a_path) {
        return i_tree == a_path.i_tree;
    }

    private void checkUpperNull() {
        if (i_upperNull == -1) {
            i_traverser.i_handler.prepareComparison(null);
            i_tree.compare(null);
            int[] nullMatches = i_tree.i_fieldTransaction.i_index
                .fileRangeReader().lowerAndUpperMatches();
            if (nullMatches[0] == 0) {
                i_upperNull = nullMatches[1] + 1;
            } else {
            	i_upperNull = 0; 
            }
        }
    }

    int countMatching() {
        if (i_comparisonResult == 0) {
            if (i_lowerAndUpperMatch == null) {
                if (i_tree instanceof IxRemove) {
                    return 0;
                }
                return 1;
            }
            return i_lowerAndUpperMatch[1] - i_lowerAndUpperMatch[0] + 1;
        }
        return 0;
    }

    int countPreceding(boolean a_takenulls) {
        int preceding = 0;
        if (i_tree.i_preceding != null) {
            if (i_next == null || i_next.i_tree != i_tree.i_preceding) {
                preceding += i_tree.i_preceding.i_size;
            }
        }
        if (i_lowerAndUpperMatch != null) {
            if(a_takenulls) {
                i_upperNull = 0;
            }else {
                checkUpperNull();
            }
            preceding += i_lowerAndUpperMatch[0] - i_upperNull;
        } else {
            if (i_comparisonResult < 0 && !(i_tree instanceof IxRemove)) {
                preceding++;
            }
        }
        return preceding;
    }

    int countSubsequent() {
        int subsequent = 0;
        if (i_tree.i_subsequent != null) {
            if (i_next == null || i_next.i_tree != i_tree.i_subsequent) {
                subsequent += i_tree.i_subsequent.i_size;
            }
        }
        if (i_lowerAndUpperMatch != null) {
            subsequent += ((IxFileRange) i_tree).i_entries
                - i_lowerAndUpperMatch[1] - 1;
        } else {
            if (i_comparisonResult > 0 && !(i_tree instanceof IxRemove)) {
                subsequent++;
            }
        }
        return subsequent;
    }

    IxPath shallowClone() {
        try {
            return (IxPath) this.clone();
        } catch (CloneNotSupportedException e) {

        }
        return null;
    }

    public String toString() {
        return i_tree.toString();
    }

    public void visit(Object a_object) {
        i_candidatesTree = ((IxTree) a_object).addToCandidatesTree(
            i_candidatesTree, i_candidates, null);
    }

}