/* Copyright (C) 2004   db4objects Inc.   http://www.db4o.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA  02111-1307, USA. */

package com.db4o;

import com.db4o.ext.*;

/**
 * the db4o server class. 
 * @see <a href="Db4o.html#openServer(java.lang.String, int)">
 * <code>Db4o.openServer()</code></a>
 * @see <a href="ext/ExtObjectServer.html">ExtObjectServer</a> for
 * extended functionality.
 */
public interface ObjectServer {
    /**
    * closes the <code>ObjectServer</code> and writes all cached data.
    * <br><br>
    * @return true - denotes that the last instance connected to the
    * used database file was closed.
    */
    public boolean close();

    /**
     * returns an ObjectServer with extended functionality.
     * <br><br>Use this method as a convient accessor to extended methods. 
     * Every ObjectServer can be casted to an ExtObjectServer.
     * <br><br>The functionality is split to two interfaces to allow newcomers to
     * focus on the essential methods.
     */
    public ExtObjectServer ext();

    /**
     * grants client access to the specified user with the specified password.
     * <br><br>If the user already exists, the password is changed to 
     * the specified password.<br><br>
     * @param userName the name of the user
     * @param password the password to be used
     */
    public void grantAccess(String userName, String password);

    /**
     * opens a client against this server.
     * <br><br>A client opened with this method operates within the same VM
     * as the server. Since an embedded client can use direct communication, without
     * an in-between socket connection, performance will be better than a client
     * opened with 
     * {@link Db4o#openClient(java.lang.String, int, java.lang.String, java.lang.String)}
     * 
     * <br><br>Every client has it's own transaction and uses it's own cache
     * for it's own version of all peristent objects.  
     * 
     */
    public ObjectContainer openClient();

}
