/* Copyright (C) 2004   db4objects Inc.   http://www.db4o.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA  02111-1307, USA. */

package com.db4o;

import java.lang.reflect.*;

import com.db4o.ext.*;

/**
 * database boot record. Responsible for ID generation, version generation and
 * holding a reference to the Db4oDatabase object of the ObjectContainer
 */
class PBootRecord extends P1Object implements Db4oTypeImpl{

    transient YapFile         i_stream;
    public Db4oDatabase       i_db;
    public long               i_uuidGenerator;
    public long               i_versionGenerator;
    public int                i_generateVersionNumbers;
    public int                i_generateUUIDs;
    private transient boolean i_dirty;

    public MetaIndex          i_uuidMetaIndex;
    
    public PBootRecord(){
    }

    public int activationDepth() {
        return Integer.MAX_VALUE;
    }

    void init(Config4Impl a_config) {
        i_db = Db4oDatabase.generate();
        i_uuidGenerator = Unobfuscated.randomLong();
        initConfig(a_config);
        i_dirty = true;
    }

    boolean initConfig(Config4Impl a_config) {
        boolean modified = false;
        Class myClass = this.getClass();
        Class configClass = a_config.getClass();
        Field[] fields = myClass.getDeclaredFields();
        for (int i = 0; i < fields.length; i++) {
            try {
                Field field = configClass.getField(fields[i].getName());
                if (field != null) {
                    Object obj = field.get(a_config);
                    if (obj != null) {
                        YapClass yc = i_stream.i_handlers.getYapClassStatic(obj
                            .getClass());
                        if (yc instanceof YapClassPrimitive) {
                            YapJavaClass yjc = (YapJavaClass) ((YapClassPrimitive) yc).i_handler;
                            if (!yjc.primitiveNull().equals(obj)) {
                                fields[i].set(this, obj);
                                modified = true;
                            }
                        }
                    }
                }
            } catch (Exception e) {
                // e.printStackTrace();
            }
        }
        return modified;
    }
    
    MetaIndex getUUIDMetaIndex(){
        if (i_uuidMetaIndex == null) {
            i_uuidMetaIndex = new MetaIndex();
            Transaction systemTrans = i_stream.getSystemTransaction();
            i_stream.setInternal(systemTrans, this, false);
            systemTrans.commit();
        }
        return i_uuidMetaIndex;
    }

    long newUUID() {
        i_dirty = true;
        return i_uuidGenerator++;
    }
    
    void setDirty(){
        i_dirty = true;
    }

    void store(int a_depth) {
        if (i_dirty) {
            i_versionGenerator++;
            super.store(a_depth);
        }
        i_dirty = false;
    }

    long version() {
        i_dirty = true;
        return i_versionGenerator;
    }

}