/* Copyright (C) 2004   db4objects Inc.   http://www.db4o.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA  02111-1307, USA. */

package com.db4o;

public class QEvaluation extends QCon {

    private transient Object i_evaluation;
    byte[] i_marshalledEvaluation;
    int i_marshalledID;
    boolean i_isDelegate;

    public QEvaluation() {
        // C/S only
    }

    QEvaluation(Transaction a_trans, Object a_evaluation, boolean a_isDelegate) {
        super(a_trans);
        i_evaluation = a_evaluation;
        i_isDelegate = a_isDelegate;
    }

    void evaluateEvaluationsExec(QCandidates a_candidates, boolean rereadObject) {
        if (rereadObject) {
            a_candidates.traverse(new Visitor4() {
                public void visit(Object a_object) {
                    ((QCandidate) a_object).useField(null);
                }
            });
        }
        a_candidates.filter(this);
    }

    void marshall() {
        super.marshall();
		int[] id = {0};
		if(Deploy.csharp){
		    i_marshalledEvaluation = i_trans.i_stream.marshall(i_evaluation, id);
		}else{
		    try{
		        // try serialisation. If it fails, store as db4o.
		        i_marshalledEvaluation = Platform.serialize(i_evaluation);
		    }catch (Exception e){
		        i_marshalledEvaluation = i_trans.i_stream.marshall(i_evaluation, id);
		    }
		    
		}
		i_marshalledID = id[0];
    }

    void unmarshall(Transaction a_trans) {
        if (i_trans == null) {
            super.unmarshall(a_trans);
            if(Deploy.csharp){
                i_evaluation = i_trans.i_stream.unmarshall(i_marshalledEvaluation, i_marshalledID);
            }else{
                if(i_marshalledID > 0){
                    i_evaluation = i_trans.i_stream.unmarshall(i_marshalledEvaluation, i_marshalledID);
                }else{
                    i_evaluation = Platform.deserialize(i_marshalledEvaluation);
                }
            }
        }
    }

    public void visit(Object obj) {
		QCandidate candidate = (QCandidate)obj;		
        try {
            Platform.evaluationEvaluate(i_evaluation, candidate);
            if(! candidate.i_include){
            	doNotInclude(candidate.getRoot());
            }
        } catch (Exception e) {
            candidate.include(false);
			doNotInclude(candidate.getRoot());
            // TODO: implement Exception callback for the user coder
            // at least for test cases
        }
    }
    
    boolean supportsIndex(){
        return false;
    }
}
