/* Copyright (C) 2004   db4objects Inc.   http://www.db4o.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA  02111-1307, USA. */

package com.db4o;

public class QField implements Visitor4{
	
	transient Transaction i_trans;
	String i_name;
	transient YapField i_yapField;
	int i_yapClassID;
	int i_index;
	
	public QField(){
		// C/S only	
	}
	
	QField(Transaction a_trans, String name, YapField a_yapField, int a_yapClassID, int a_index){
		i_trans = a_trans;
		i_name = name;
		i_yapField = a_yapField;
		i_yapClassID = a_yapClassID;
		i_index = a_index;
		if(i_yapField != null){
		    if(! i_yapField.alive()){
		        i_yapField = null;
		    }
		}
	}
	
	boolean canHold(Object a_object){
	    Class clazz = null;
	    if(a_object != null){
	        if(a_object instanceof Class){
	            clazz = (Class)a_object;
	        }else{
	            clazz = a_object.getClass();
	        }
	    }else{
			if(Deploy.csharp){
				return true;
			}
	    }
		return i_yapField == null || i_yapField.canHold(clazz);
	}
	
	YapClass getYapClass(){
		if(i_yapField != null){
			return i_yapField.getFieldYapClass(i_trans.i_stream);
		}
		return null;
	}
	
	YapField getYapField(YapClass yc){
		if(i_yapField != null){
			return i_yapField;
		}
		YapField yf = yc.getYapField(i_name);
		if(yf != null){
		    yf.alive();
		}
		return yf;
	}
	
	boolean isArray(){
		return i_yapField != null && i_yapField.getHandler() instanceof YapArray;
	}
	
	boolean isClass(){
		return i_yapField == null || i_yapField.getHandler().getType() == YapConst.TYPE_CLASS;
	}
	
	boolean isSimple(){
		return i_yapField != null && i_yapField.getHandler().getType() == YapConst.TYPE_SIMPLE;
	}
	
	YapComparable prepareComparison(Object obj){
		if(i_yapField != null){
			return i_yapField.prepareComparison(obj);
		}
		if(obj == null){
			return Null.INSTANCE;
		}
		YapClass yc = i_trans.i_stream.getYapClass(obj.getClass(), true);
		YapField yf = yc.getYapField(i_name);
		if(yf != null){
			return yf.prepareComparison(obj);
		}
		return null;
	}
	
	void unmarshall(Transaction a_trans){
		if(i_yapClassID != 0){
			YapClass yc = a_trans.i_stream.getYapClass(i_yapClassID);
			i_yapField = yc.i_fields[i_index];
		}
	}
	
	public void visit(Object obj) {
		((QCandidate) obj).useField(this);
	}
}

