/* Copyright (C) 2004   db4objects Inc.   http://www.db4o.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA  02111-1307, USA. */

package com.db4o;
  
public class TreeInt extends Tree implements ReadWriteable{
	
	int i_key;
	
	public TreeInt(int a_key){
		this.i_key = a_key;
	}

	int compare(Tree a_to) {
		return i_key - ((TreeInt)a_to).i_key;
	}
	
	Tree deepClone(){
		return new TreeInt(i_key);
	}
	
	boolean duplicates(){
		return false;
	}
	
	static final TreeInt find(Tree a_in, int a_key){
		if(a_in == null){
			return null;
		}
		return ((TreeInt)a_in).find(a_key);
	}
	
	final TreeInt find(int a_key){
		int cmp = i_key - a_key;
		if (cmp < 0){
			if(i_subsequent != null){
				return ((TreeInt)i_subsequent).find(a_key);
			}
		}else{
			if (cmp > 0){
				if(i_preceding != null){
					return ((TreeInt)i_preceding).find(a_key);
				}
			}else{
				return this;
			}
		}
		return null;
	}

	
	public Object read(YapReader a_bytes){
		return new TreeInt(a_bytes.readInt());
	}
	
	public void write(YapWriter a_writer){
		a_writer.writeInt(i_key);
	}

	int ownLength(){
		return YapConst.YAPINT_LENGTH;
	}
	
	boolean variableLength(){
		return false;
	}
	
	QCandidate toQCandidate(QCandidates candidates){
		QCandidate qc = new QCandidate(candidates, i_key, true);
		qc.i_preceding = toQCandidate((TreeInt)i_preceding, candidates); 
		qc.i_subsequent = toQCandidate((TreeInt)i_subsequent, candidates);
		qc.i_size = i_size; 
		return qc; 
	}
	
	static QCandidate toQCandidate(TreeInt tree, QCandidates candidates){
		if(tree == null){
			return null;
		}
		return tree.toQCandidate(candidates);
	}
	
	
	
}
