/* Copyright (C) 2004   db4objects Inc.   http://www.db4o.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA  02111-1307, USA. */

package com.db4o;

import com.db4o.reflect.*;

final class YapConstructor {
    private final Class i_javaClass;
    private IClass i_reflectorClass;

    private final EventDispatcher i_eventDispatcher;

    private final IConstructor i_constructor;
    private final Object[] i_params;

    YapConstructor(
        YapStream a_stream,
        Class a_class,
        IConstructor a_constructor,
        Object[] a_params,
        boolean a_checkDispatcher) {
        
        i_javaClass = a_class;
        i_constructor = a_constructor;
        i_params = a_params;
	        try {
	            if (a_stream == null || a_stream.i_config.i_reflect instanceof CReflect) {
	                i_reflectorClass = new CClass(a_class);
	            } else {
	                i_reflectorClass = a_stream.i_config.i_reflect.forName(a_class.getName());
	            }
	        } catch (ClassNotFoundException e) {
	        }
	        
	        i_eventDispatcher = a_checkDispatcher ? EventDispatcher.forClass(a_stream, i_reflectorClass) : null;
    }

    boolean dispatch(YapStream a_stream, Object obj, int eventID) {
        if (i_eventDispatcher != null) {
            return i_eventDispatcher.dispatch(a_stream, obj, eventID);
        }
        return true;
    }

    String getName() {
        return i_javaClass.getName();
    }

    Class javaClass() {
        return i_javaClass;
    }

    Object newInstance() throws Exception {
        if (i_constructor == null) {
            return i_reflectorClass.newInstance();
        } else {
            return i_constructor.newInstance(i_params);
        }

    }

    IClass reflectorClass() {
        return i_reflectorClass;
    }
}
