/* Copyright (C) 2004   db4objects Inc.   http://www.db4o.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA  02111-1307, USA. */

package com.db4o;

/**
 * 
 */
abstract class YapFieldVirtual extends YapField {

    static final String PREFIX = "v4o";

    YapFieldVirtual() {
        super(null);
    }
    
    void addFieldIndex(YapWriter a_writer, boolean a_new) {
        a_writer.incrementOffset(linkLength());
    }
    
    void delete(YapWriter a_bytes) {
        a_bytes.incrementOffset(linkLength());
    }

    int ownLength(YapStream a_stream) {
        return a_stream.i_stringIo.shortLength(i_name);
    }

    void initIndex(YapStream a_stream, MetaIndex a_metaIndex) {
        if (i_index == null) {
            i_index = new IxField(a_stream.getSystemTransaction(), this,
                a_metaIndex);
        }
    }

    void instantiate(YapObject a_yapObject, Object a_onObject, YapWriter a_bytes)
        throws CorruptionException {
        if (a_yapObject.i_virtualAttributes == null) {
            a_yapObject.i_virtualAttributes = new VirtualAttributes();
        }
        instantiate1(a_bytes.getTransaction(), a_yapObject, a_bytes);
    }

    abstract void instantiate1(Transaction a_trans, YapObject a_yapObject, YapReader a_bytes);
    
    void loadHandler(YapStream a_stream){
    	// do nothing
    }

    void marshall(YapObject a_yapObject, Object a_object, YapWriter a_bytes,
        Config4Class a_config, boolean a_new) {
        YapStream stream = a_bytes.i_trans.i_stream;
        boolean migrating = false;
        if (stream instanceof YapFile) {
	        if (stream.i_migrateFrom != null) {
	            migrating = true;
	            if (a_yapObject.i_virtualAttributes == null) {
	                YapObject migrateYapObject = stream.i_migrateFrom
	                    .getYapObject(a_yapObject.getObject());
	                if (migrateYapObject != null
	                    && migrateYapObject.i_virtualAttributes != null
	                    && migrateYapObject.i_virtualAttributes.i_database != null) {
	                    migrating = true;
	                    a_yapObject.i_virtualAttributes = migrateYapObject.i_virtualAttributes
	                        .shallowClone();
	                    a_yapObject.i_virtualAttributes.i_database = stream.i_handlers.ensureDb4oDatabase(a_bytes.getTransaction(),migrateYapObject.i_virtualAttributes.i_database );
	                }
	            }
	        }
	        if (a_yapObject.i_virtualAttributes == null) {
	            a_yapObject.i_virtualAttributes = new VirtualAttributes();
	            migrating = false;
	        }
        }else{
            migrating = true;
        }
	    marshall1(a_yapObject, a_bytes, migrating, a_new);
    }

    abstract void marshall1(YapObject a_yapObject, YapWriter a_bytes,
        boolean a_migrating, boolean a_new);
    
    public void readVirtualAttribute(Transaction a_trans, YapReader a_reader, YapObject a_yapObject) {
        instantiate1(a_trans, a_yapObject, a_reader);
    }

    void writeThis(YapWriter a_writer, YapClass a_onClass) {
        a_writer.writeShortString(i_name);
    }
}