/* Copyright (C) 2004   db4objects Inc.   http://www.db4o.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA  02111-1307, USA. */

package com.db4o.config;

import com.db4o.*;

/**
 * interface to allow instantiating objects by calling specific constructors.
 * <br><br><b>Examples: ../com/db4o/samples/translators.</b><br><br>
 * By writing classes that implement this interface, it is possible to
 * define which constructor is to be used during the instantiation of a stored object.
 * <br><br>
 * Before starting a db4o session, translator classes that implement the 
 * <code>ObjectConstructor</code> or 
 * <a href="ObjectTranslator.html"><code>ObjectTranslator</code></a>
 * need to be registered.<br><br>
 * Example:<br>
 * <code>
 * Configuration config = Db4o.configure();<br>
 * ObjectClass oc = config.objectClass("package.className");<br>
 * oc.translate(new FooTranslator());</code><br><br>
 */
public interface ObjectConstructor extends ObjectTranslator {

	/**
	 * db4o calls this method when a stored object needs to be instantiated.
	 * <br><br>
	 * @param container the ObjectContainer used
	 * @param storedObject the object stored with 
	 * <a href="ObjectTranslator.html#onStore(com.db4o.ObjectContainer, java.lang.Object)">
	 * <code>ObjectTranslator.html#onStore()</code></a>.
	 * @return the instantiated object.
	 */
	public Object onInstantiate(ObjectContainer container, Object storedObject);
	
}