/* Copyright (C) 2004   db4objects Inc.   http://www.db4o.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA  02111-1307, USA. */

package com.db4o.ext;

import com.db4o.*;
import com.db4o.types.*;


/**
 * Class to identify a database by it's signature.
 * <br><br>db4o UUID handling uses a reference to the Db4oDatabase object, that
 * represents the database an object was created on.  
 */
public class Db4oDatabase implements Db4oType{

    /**
     * Field is public for implementation reasons, DO NOT TOUCH!
     */
    public byte[] i_signature;
    
    /**
     * Field is public for implementation reasons, DO NOT TOUCH!
     */
    public long i_uuid;
    
    /**
     * cached ObjectContainer for getting the own ID.
     */
    private transient ExtObjectContainer i_objectContainer;
    
    /**
     * cached ID, only valid in combination with i_objectContainer
     */
    private transient int i_id; 
    
    
    /**
     * generates a new Db4oDatabase object with a unique signature.
     */
    public static Db4oDatabase generate() {
        Db4oDatabase db = new Db4oDatabase();
        db.i_signature = Unobfuscated.generateSignature();
        db.i_uuid = System.currentTimeMillis();
        return db;
    }
    
    /**
     * comparison by signature.
     */
    public boolean equals(Object obj) {
        if(obj instanceof Db4oDatabase) {
            Db4oDatabase other = (Db4oDatabase)obj;
            if(other.i_signature.length == i_signature.length) {
                for (int i = 0; i < i_signature.length; i++) {
                    if(i_signature[i] != other.i_signature[i]) {
                        return false;
                    }
                }
                return true;
            }
        }
        return false;
    }
    
    /**
     * gets the db4o ID, and may cache it for performance reasons.
     * @param a_oc the ObjectContainer
     * @return the db4o ID for the ObjectContainer
     */
    public int getID(ExtObjectContainer a_oc) {
        if(a_oc != i_objectContainer) {
            i_objectContainer = a_oc;
            i_id = (int)a_oc.getID(this);
            if(i_id == 0){
                a_oc.set(this);
                i_id = (int)a_oc.getID(this);
            }
        }
        return i_id;
    }
    
    public String toString(){
        return "Db4oDatabase: " + i_signature;
    }
    
}