/* Copyright (C) 2004   db4objects Inc.   http://www.db4o.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA  02111-1307, USA. */

package com.db4o.reflect;

/**
 * root of the reflection implementation API.
 * <br><br>The open reflection interface is supplied to allow to implement
 * reflection functionality on JDKs that do not come with the
 * java.lang.reflect.* package.<br><br>
 * See the code in com.db4o.samples.reflect for a reference implementation
 * that uses java.lang.reflect.*.
 * <br><br>
 * Use <a href="../config/Configuration.html#reflectWith(com.db4o.reflect.IReflect)">
 * <code>Db4o.configure().reflectWith(IReflect reflector)</code></a>
 * to register the use of your implementation before opening database
 * files.
 */
public interface IReflect {
	
	
	/**
	 * returns an IArray object, the equivalent to java.lang.reflect.Array.
	 */
	public IArray array();
	
	/**
	 * specifiy whether parameterized Constructors are supported.
	 * <br><br>The support of Constructors is optional. If Constructors
	 * are not supported, every persistent class needs a public default
	 * constructor with zero parameters.
	 */
	public boolean constructorCallsSupported();
	
	/**
	 * returns an IClass class reflector for a class name.
	 */
	public IClass forName(String className) throws ClassNotFoundException;
	
	
	/**
	 * specify whether method calls are supported.
	 */
	public boolean methodCallsSupported();
	
}
