/* Copyright (C) 2004   db4objects Inc.   http://www.db4o.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA  02111-1307, USA. */

package com.db4o.test;

import com.db4o.ext.*;

public class SetSemaphore {

    public void test() {

        final ExtObjectContainer client1 = Test.objectContainer();

        Test.ensure(client1.setSemaphore("hi", 0));
        Test.ensure(client1.setSemaphore("hi", 0));

        if (Test.clientServer) {
            final ExtObjectContainer client2 = Test.open();
            final ExtObjectContainer client3 = Test.open();
            final ExtObjectContainer client4 = Test.open();
            final ExtObjectContainer client5 = Test.open();

            Test.ensure(!client2.setSemaphore("hi", 0));
            client1.releaseSemaphore("hi");
            Test.ensure(client2.setSemaphore("hi", 50));
            Test.ensure(!client1.setSemaphore("hi", 0));
            Test.ensure(!client3.setSemaphore("hi", 0));
            
            new GetAndRelease(client3);
            new GetAndRelease(client2);
            new GetAndRelease(client1);
            new GetAndRelease(client4);
            new GetAndRelease(client5);

            try {
                Thread.sleep(1000);
            } catch (Exception e) {
                e.printStackTrace();
            }
            Test.ensure(client1.setSemaphore("hi", 0));
            client1.close();
            
            new GetAndRelease(client3);
            new GetAndRelease(client2);
            try {
                Thread.sleep(1000);
            } catch (Exception e) {
                e.printStackTrace();
            }
            
            client2.close(); 
            client3.close(); // the last one opened remains
            client4.close(); // open for other tests
            
            client5.setSemaphore("hi", 1000);
        }

    }

    static class GetAndRelease implements Runnable {

        ExtObjectContainer client;

        public GetAndRelease(ExtObjectContainer client) {
            this.client = client;
            new Thread(this).start();
        }

        public void run() {
            long time = System.currentTimeMillis();
            Test.ensure(client.setSemaphore("hi", 50000));
            time = System.currentTimeMillis() - time;
            // System.out.println("Time to get semaphore: " + time);
            try {
                Thread.sleep(50);
            } catch (Exception e) {
                e.printStackTrace();
            }
            // System.out.println("About to release semaphore.");
            client.releaseSemaphore("hi");
        }
    }

}
