package com.db4o.f1.chapter2;

import java.io.File;

import com.db4o.Db4o;
import com.db4o.ObjectContainer;
import com.db4o.ObjectSet;
import com.db4o.f1.Util;
import com.db4o.query.Query;


public class StructuredExample {
    private final static String FILENAME="f1.yap";
    
    public static void main(String[] args) {
        new File(FILENAME).delete();
        ObjectContainer db=Db4o.openFile(FILENAME);
        try {
            storeFirstCar(db);
            storeSecondCar(db);
            retrieveAllCarsQBE(db);
            retrieveAllPilotsQBE(db);
            retrieveCarByPilotQBE(db);
            retrieveCarByPilotNameQuery(db);
            retrieveCarByPilotProtoQuery(db);
            updateCar(db);
            updatePilotSingleSession(db);
            updatePilotSeparateSessionsPart1(db);
            db.close();
            db=Db4o.openFile(FILENAME);
            updatePilotSeparateSessionsPart2(db);
            db.close();
            updatePilotSeparateSessionsImprovedPart1();
            db=Db4o.openFile(FILENAME);
            updatePilotSeparateSessionsImprovedPart2(db);
            db.close();
            db=Db4o.openFile(FILENAME);
            updatePilotSeparateSessionsImprovedPart3(db);
            deleteFlat(db);
            db.close();
            deleteDeepPart1();
            db=Db4o.openFile(FILENAME);
            deleteDeepPart2(db);
            deleteDeepRevisited(db);
        }
        finally {
            db.close();
        }
    }
    
    public static void storeFirstCar(ObjectContainer db) {
        Car car1=new Car("Ferrari");
        Pilot pilot1=new Pilot("Michael Schumacher",100);
        car1.setPilot(pilot1);
        db.set(car1);
    }

    public static void storeSecondCar(ObjectContainer db) {
        Pilot pilot2=new Pilot("Rubens Barrichello",99);
        db.set(pilot2);
        Car car2=new Car("BMW");
        car2.setPilot(pilot2);
        db.set(car2);
    }

    public static void retrieveAllCarsQBE(ObjectContainer db) {
        Car proto=new Car(null);
        ObjectSet result=db.get(proto);
        Util.listResult(result);
    }

    public static void retrieveAllPilotsQBE(ObjectContainer db) {
        Pilot proto=new Pilot(null,0);
        ObjectSet result=db.get(proto);
        Util.listResult(result);
    }

    public static void retrieveCarByPilotQBE(
            ObjectContainer db) {
        Pilot pilotproto=new Pilot("Rubens Barrichello",0);
        Car carproto=new Car(null);
        carproto.setPilot(pilotproto);
        ObjectSet result=db.get(carproto);
        Util.listResult(result);
    }
    
    public static void retrieveCarByPilotNameQuery(
            ObjectContainer db) {
        Query query=db.query();
        query.constrain(Car.class);
        query.descend("pilot").descend("name")
                .constrain("Rubens Barrichello");
        ObjectSet result=query.execute();
        Util.listResult(result);
    }

    public static void retrieveCarByPilotProtoQuery(
                ObjectContainer db) {
        Query query=db.query();
        query.constrain(Car.class);
        Pilot proto=new Pilot("Rubens Barrichello",0);
        query.descend("pilot").constrain(proto);
        ObjectSet result=query.execute();
        Util.listResult(result);
    }
    
    public static void updateCar(ObjectContainer db) {
        ObjectSet result=db.get(new Car("Ferrari"));
        Car found=(Car)result.next();
        found.setPilot(new Pilot("Somebody else",0));
        db.set(found);
        result=db.get(new Car("Ferrari"));
        Util.listResult(result);
    }

    public static void updatePilotSingleSession(
                ObjectContainer db) {
        ObjectSet result=db.get(new Car("Ferrari"));
        Car found=(Car)result.next();
        found.getPilot().addPoints(1);
        db.set(found);
        result=db.get(new Car("Ferrari"));
        Util.listResult(result);
    }

    public static void updatePilotSeparateSessionsPart1(
                ObjectContainer db) {
    	ObjectSet result=db.get(new Car("Ferrari"));
        Car found=(Car)result.next();
        found.getPilot().addPoints(1);
        db.set(found);
    }

    public static void updatePilotSeparateSessionsPart2(
                ObjectContainer db) {
        ObjectSet result=db.get(new Car("Ferrari"));
        Util.listResult(result);
    }

    public static void updatePilotSeparateSessionsImprovedPart1() {
        Db4o.configure().objectClass("com.db4o.f1.chapter2.Car")
                .cascadeOnUpdate(true);        
    }

    public static void updatePilotSeparateSessionsImprovedPart2(
                ObjectContainer db) {
        ObjectSet result=db.get(new Car("Ferrari"));
        Car found=(Car)result.next();
        found.getPilot().addPoints(1);
        db.set(found);
    }

    public static void updatePilotSeparateSessionsImprovedPart3(
                ObjectContainer db) {
        ObjectSet result=db.get(new Car("Ferrari"));
        Util.listResult(result);
    }

    public static void deleteFlat(ObjectContainer db) {
        ObjectSet result=db.get(new Car("Ferrari"));
        Car found=(Car)result.next();
        db.delete(found);
        result=db.get(new Car(null));
        Util.listResult(result);
    }
    
    public static void deleteDeepPart1() {
        Db4o.configure().objectClass("com.db4o.f1.chapter2.Car")
                .cascadeOnDelete(true);
    }

    public static void deleteDeepPart2(ObjectContainer db) {
        ObjectSet result=db.get(new Car("BMW"));
        Car found=(Car)result.next();
        db.delete(found);
        result=db.get(new Car(null));
        Util.listResult(result);
    }

    public static void deleteDeepRevisited(ObjectContainer db) {
        ObjectSet result=db.get(new Pilot("Michael Schumacher",0));
        Pilot pilot=(Pilot)result.next();
        Car car1=new Car("Ferrari");
        Car car2=new Car("BMW");
        car1.setPilot(pilot);
        car2.setPilot(pilot);
        db.set(car1);
        db.set(car2);
        db.delete(car2);
        result=db.get(new Car(null));
        Util.listResult(result);
    }
    
    public static void deleteAll(ObjectContainer db) {
        ObjectSet cars=db.get(new Car(null));
        while(cars.hasNext()) {
            db.delete(cars.next());
        }
        ObjectSet pilots=db.get(new Pilot(null,0));
        while(pilots.hasNext()) {
            db.delete(pilots.next());
        }
    }
}
